package cn.iocoder.yudao.module.sale.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.*;
import cn.iocoder.yudao.framework.apollo.core.event.export.base.CustomerExportEvent;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.customer.convert.customer.CustomerConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.indirectCustomer.IndirectCustomerService;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerExcelVO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerExportReqVO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;


/**
 * 客户excel导出监听
 *
 * @author zhengYi
 */
@Component("CustomerExcelExportListener")
@AllArgsConstructor
@Slf4j
public class CustomerExcelExportListener {


    private UeProperties ueProperties;
    private FileService fileService;


    private final CustomerService customerService;

    /**
     * 客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerExcelExportPushEvent.class)
    public void customerExcelExportPushEvent(CustomerExcelExportPushEvent event) {

        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }
    /**
     * 我的客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerMineExcelExportPushEvent.class)
    public void customerMineExcelExportPushEvent(CustomerMineExcelExportPushEvent event) {
        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }

    /**
     * 我的待接收客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerWaitConfirmExcelExportPushEvent.class)
    public void customerWaitConfirmExcelExportPushEvent(CustomerWaitConfirmExcelExportPushEvent event) {

        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getWaitToConfirmCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }

    /**
     * 公海池客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerPublicExcelExportPushEvent.class)
    public void customerPublicExcelExportPushEvent(CustomerPublicExcelExportPushEvent event) {
        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getPublicCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }

    /**
     * 待分配客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerWaitDistributionExcelExportPushEvent.class)
    public void customerWaitDistributionExcelExportPushEvent(CustomerWaitDistributionExcelExportPushEvent event) {
        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getWaitToAssignedCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }


    /**
     * 潜在客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerPotentialExcelExportPushEvent.class)
    public void customerPotentialExcelExportPushEvent(CustomerPotentialExcelExportPushEvent event) {
        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }


    /**
     * 部门客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerDeptExcelExportPushEvent.class)
    public void customerDeptExcelExportPushEvent(CustomerDeptExcelExportPushEvent event) {
        CustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerExportReqVO.class);
        List<CustomerDO> list = customerService.getDeptCustomerExcelVoList(exportReqVO);

        makeExcelUpload(event, list);
    }




    public void makeExcelUpload(CustomerExportEvent event, List<CustomerDO> customerDOList) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                if(CollectionUtil.isEmpty(customerDOList)) return;


                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/customer/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("customer.xlsx");


                // 导出 Excel
                List<CustomerExcelVO> datas = CustomerConvert.INSTANCE.convertList02(customerDOList);

                String path = ExcelUtils.write(dir, fileName, "数据", CustomerExcelVO.class, datas);

                // 获取到临时文件
                File file = new File(path);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());

            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }




}
